package cz.cvut.kbss.reporting.security.oauth2.google;

import cz.cvut.kbss.reporting.model.Person;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.oauth2.client.OAuth2RestOperations;
import org.springframework.security.oauth2.client.OAuth2RestTemplate;
import org.springframework.security.oauth2.client.resource.BaseOAuth2ProtectedResourceDetails;
import org.springframework.security.oauth2.common.AuthenticationScheme;
import org.springframework.security.oauth2.common.DefaultOAuth2AccessToken;
import org.springframework.security.oauth2.common.OAuth2AccessToken;
import org.springframework.security.oauth2.common.exceptions.InvalidTokenException;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.security.oauth2.provider.OAuth2Request;
import org.springframework.security.oauth2.provider.token.ResourceServerTokenServices;

public class GoogleUserInfoTokenServices implements ResourceServerTokenServices {

    protected final Logger logger = LoggerFactory.getLogger(this.getClass());

    private final String userInfoEndpointUrl;

    private final String clientId;

    private String tokenType = DefaultOAuth2AccessToken.BEARER_TYPE;

    public GoogleUserInfoTokenServices(String userInfoEndpointUrl, String clientId) {
        this.userInfoEndpointUrl = userInfoEndpointUrl;
        this.clientId = clientId;
    }

    @Override
    public OAuth2Authentication loadAuthentication(String accessToken)
        throws AuthenticationException, InvalidTokenException {
        Map<String, Object> map = getMap(this.userInfoEndpointUrl, accessToken);
        if (map.containsKey("error")) {
            this.logger.debug("userinfo returned error: {}", map.get("error"));
            throw new InvalidTokenException(accessToken);
        }
        this.logger.debug("userinfo returned: {}", map);
        return extractAuthentication(map);
    }

    private OAuth2Authentication extractAuthentication(Map<String, Object> map) {
        UserDetails principal = getUserDetails(map);
        List<GrantedAuthority> authorities =
            new ArrayList<>(principal.getAuthorities());
        OAuth2Request request =
            new OAuth2Request(null, this.clientId, authorities, true, null, null, null, null, null);
        UsernamePasswordAuthenticationToken token =
            new UsernamePasswordAuthenticationToken(principal, "N/A", authorities);
        token.setDetails(map);
        return new OAuth2Authentication(request, token);
    }

    private UserDetails getUserDetails(Map<String, Object> map) {
        final Person person = new Person();
        person.setUsername((String) map.get("name"));
        person.setFirstName((String) map.get("given_name"));
        person.setLastName((String) map.get("family_name"));
        return new cz.cvut.kbss.reporting.security.model.UserDetails(person);
    }

    @SuppressWarnings( {"unchecked"})
    private Map<String, Object> getMap(String path, String accessToken) {
        this.logger.info("Getting user info from: {}", path);
        try {
            BaseOAuth2ProtectedResourceDetails resource = new BaseOAuth2ProtectedResourceDetails();
            resource.setAuthenticationScheme(AuthenticationScheme.query);
            OAuth2RestOperations restTemplate = new OAuth2RestTemplate(resource);
            OAuth2AccessToken existingToken =
                restTemplate.getOAuth2ClientContext().getAccessToken();
            if (existingToken == null || !accessToken.equals(existingToken.getValue())) {
                DefaultOAuth2AccessToken token = new DefaultOAuth2AccessToken(accessToken);
                token.setTokenType(this.tokenType);
                restTemplate.getOAuth2ClientContext().setAccessToken(token);
            }
            return restTemplate.getForEntity(path, Map.class).getBody();
        } catch (Exception ex) {
            this.logger
                .info("Could not fetch user details: " + ex.getClass() + ", " + ex.getMessage());
            return Collections.singletonMap("error", "Could not fetch user details");
        }
    }

    @Override
    public OAuth2AccessToken readAccessToken(String accessToken) {
        throw new UnsupportedOperationException("Not supported: read access token");
    }
}
