'use strict';

import React from "react";
import Reflux from "reflux";
import assign from "object-assign";

import Actions from "../../actions/Actions";
import I18nWrapper from "../../i18n/I18nWrapper";
import injectIntl from "../../utils/injectIntl";
import LoadingWrapper from "../misc/LoadingWrapper";
import MessageWrapper from "../misc/MessageWrapper";
import Report from "./Report";
import ReportFactory from "../../model/ReportFactory";
import ReportStore from "../../stores/ReportStore";
import ResourceNotFound from "../ResourceNotFound";
import Routes from "../../utils/Routes";
import Routing from "../../utils/Routing";

class ReportController extends Reflux.Component {
    constructor(props) {
        super(props);
        this.i18n = props.i18n;
        this.state = {
            loading: false
        };
        this.store = ReportStore;
        this.storeKeys = ['currentReport'];
    }

    componentDidMount() {
        if (this._isNew()) {
            const report = ReportFactory.createReport();
            this.setState({currentReport: report});
        } else {
            this._loadReport();
        }
    }

    _isNew() {
        const params = this.props.match.params;
        return !params.reportId;
    }

    _loadReport() {
        this.setState({loading: true});
        Actions.loadReport(this.props.match.params.reportId, this._onLoadFinish);
        this.props.loadingOn();
    }

    _onLoadFinish = () => {
        this.props.loadingOff();
        this.setState({loading: false});
    };

    onSave = () => {
        const report = this.state.currentReport;
        if (report.isNew) {
            delete report.isNew;
            Actions.createReport(report, this._onSaveSuccess, this._onSaveError);
        } else {
            Actions.updateReport(report, this._onSaveSuccess, this._onSaveError);
        }
        this.props.loadingOn();
    };

    _onSaveSuccess = (reportId) => {
        this.props.loadingOff();
        this.props.showSuccessMessage(this.i18n('save-success-message'));
        if (reportId && reportId !== this.state.currentReport.id) {
            Routing.transitionTo(Routes.editReport, {
                params: {reportId: reportId}
            });
        }
        Actions.loadReport(reportId, this._onLoadFinish);
    };

    _onSaveError = (error) => {
        this.props.loadingOff();
        this.props.showErrorMessage(this.i18n('save-failed-message') + ' ' + error.message);
    };

    onCancel = () => {
        Routing.transitionTo(Routes.reports);
    };

    onChange = (change) => {
        const newState = assign({}, this.state.currentReport, change);
        this.setState({currentReport: newState});
    };

    render() {
        const report = this.state.currentReport;
        if (!this.state.loading && !report) {
            return <ResourceNotFound identifier={this.props.match.params.reportId}
                                     resource={this.i18n('occurrencereport.title')}/>;
        }
        return <Report report={report} save={this.onSave} cancel={this.onCancel}
                       onChange={this.onChange}/>;
    }
}

export default injectIntl(I18nWrapper(MessageWrapper(LoadingWrapper(ReportController))));
